package jip;

import java.util.Vector;

 /**<P><FONT COLOR="red">
 *<B>Descripcin:</B><BR>
 *<FONT COLOR="blue">
 *    Clase base para definir objetos funcin. Una funcin se utilizar para
 * procesar de cierto modo una imagen o secuencia de entrada devolviendo otra
 * imagen o secuencia de salida y opcionalmente otros resultados adicionales.
 * Una funcin puede o no tener parmetros o resultados asociados y conocer o
 * asignar sus valores actuales o por defecto. A los parmetros y resultados se
 * acceder por su nombre. Adems una funcin tiene asignado un nombre y una
 * descripcin de la operacin que realiza sobre la imagen de entrada.
 * No podremos crear directamente objetos de la clase funcin, tendremos que
 * crear subclases de sta que al heredar implementen como mnimo el constructor
 * (que debe asignar el nombre, la descripcin, los parmetros y los resultados)
 * y el mtodo processImg() que ser el que ante una imagen de entrada y los parmetros
 * de la funcin, devuelva una imagen de salida y los resultados adicionales.
 * @see JIPParameter
 */
public abstract class JIPFunction {

    /**
     * Variable que contendr el nombre de la funcin.
     * Slo ser accesible desde las clases derivadas que debern asignar esta
     * variable en su constructor.
     */
    protected String name;

    /**
     * Variable que contendr la descripcin de la funcin.
     * Slo ser accesible desde las clases derivadas que debern asignar esta
     * variable en su constructor.
     */
    protected String description;

    /**
     * Variable que contendr un Vector de parmetros de la funcin.
     * Slo ser accesible desde las clases derivadas que debern asignar esta
     * variable en su constructor para definir parmetros de la funcin.
     */
    protected Vector params=new Vector();

    /**
     * Variable que contendr un Vector de parmetros de salida (resultados) de la
     * funcin. Slo ser accesible desde las clases derivadas que debern asignar esta
     * variable en su constructor para definir resultados de la funcin.
     */
    protected Vector results=new Vector();

    //******************//
    //  Constructores   //
    //******************//
 
    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
		 *<FONT COLOR="blue">
     * Constructor de una funcin. La clase derivada debe implementarlo asignando
     * las variables name, description, params y results.
     */
    public JIPFunction() {}

    //*******************//
    //     METODOS       //
    //*******************//
    
    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el nombre de la funcin.
     * @return Nombre de la funcin.
     */
    public String getName() { return name; }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
  	 *<FONT COLOR="blue">
     * Obtiene la descripcin de la funcin.
     * @return Descripcin de la funcin.
     */
    public String getDescription() { return description; }


    //***********************************
    // METODOS QUE IMPLEMENTAN LA FUNCION
    //***********************************

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     *     Obtiene una imagen de salida a partir de una de entrada. Este mtodo lo
     * debe implementar la clase derivada y es el que diferenciar unas funciones
     * de otras. Si la funcin tiene parmetros se debern asignar antes los valores
     * de los requeridos como mnimo, de forma que este mtodo pueda acceder
     * a los valores correctos de los mismos. Si la funcin tiene resultados debern
     * asignarse sus valores en este mtodo de forma que posteriormente se pueda
     * acceder a los valores correctos de los mismos.
     * @param img Imagen de entrada.
     * @return Imagen de salida.
     */
    public abstract JIPImage processImg(JIPImage img);

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene una secuencia de salida a partir de una de entrada.
     * Este mtodo por defecto obtiene la secuencia de salida aplicando una a una
     * el mtodo processImg() a cada una de las imgenes de la secuencia de
     * entrada. En el caso de las imgenes de entrada en las que processImg()
     * devuelva null, la imagen en la secuencia de salida ser una copia de la
     * de entrada. Este comportamiento por defecto se puede cambiar reimplementando
     * el mtodo en la clase derivada. En el caso de que la funcin genere resultados
     * adicionales, tras ste mtodo slo tendremos los correspondientes al ltimo
	   * frame de la secuencia. Este comportamiento por defecto se puede cambiar
	   * reimplementando el mtodo para que acceda a los resultados de cada frame
	   * tras su proceso y los vaya acumulando, mostrando, almacenando, ...
     * @param seq Secuencia de entrada.
     * @return Secuencia de salida.
     */
    public JIPSequence processSeq(JIPSequence seq) {
        JIPSequence res=null;
		JIPImage img=null;
        int nf=seq.nframes;
        if(nf>0) {
            res=new JIPSequence();
            for(int i=0;i<nf;i++) {
		        img=processImg(seq.getFrame(i));
				if(img!=null) res.addFrame(img);
				else res.addFrame(new JIPImage(seq.getFrame(i)));
		    }
        }
        return res;
    }

    //*************************************
    // METODOS PARA CHEQUEAR LOS PARAMETROS
    //*************************************

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el nmero de parmetros de la funcin.
     * @return Nmero de parmetros.
     */
    public int getNumParams() {
      return params.size();
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el nmero de resultados de la funcin.
     * @return Nmero de resultados.
     */
    public int getNumResults() {
      return results.size();
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene los nombres de todos los parmetros de la funcin.
     * @return Vector con los nombres de los parmetros de la funcin.
     * null si la funcin no tiene parmetros.
     */
    public String[] getParamNames() {
      JIPParameter p=null;
      String[] res=null;
      int nparams=params.size();
      if(nparams>0) res=new String[nparams];
      for(int i=0;i<nparams;i++) {
        p=(JIPParameter)params.elementAt(i);
		res[i]=p.getName();
      }
      return res;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene los nombres de todos los resultados de la funcin.
     * @return Vector con los nombres de los resultados de la funcin.
     * null si la funcin no tiene resultados.
     */
    public String[] getResultNames() {
      JIPParameter p=null;
      String[] res=null;
      int nresults=results.size();
      if(nresults>0) res=new String[nresults];
      for(int i=0;i<nresults;i++) {
        p=(JIPParameter)results.elementAt(i);
		res[i]=p.getName();
      }
      return res;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Nos dice si existe o no un parmetro de la funcin con cierto nombre.
     * @param nom Nombre a verificar si es o no un parmetro de la funcin.
     * @return true si la funcin tiene un parmetro con nombre nom. false en
     * caso contrario.
     */
    public boolean isParam(String nom) {
      JIPParameter p=null;
      boolean res=false;
      for(int i=0;i<params.size();i++) {
        p=(JIPParameter)params.elementAt(i);
        if(nom.equals(p.getName())) {
          res=true;
          break;
        } 
      }
      return res;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Nos dice si existe o no un resultado de la funcin con cierto nombre.
     * @param nom Nombre a verificar si es o no un resultado de la funcin.
     * @return true si la funcin tiene un resultado con nombre nom. false en
     * caso contrario.
     */
    public boolean isResult(String nom) {
      JIPParameter p=null;
      boolean res=false;
      for(int i=0;i<results.size();i++) {
        p=(JIPParameter)results.elementAt(i);
        if(nom.equals(p.getName())) {
          res=true;
          break;
        }
      }
      return res;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Nos dice si un parmetro de la funcin es requerido.
     * @param nom Nombre a verificar si es o no un parmetro requerido.
     * @return true si la funcin tiene un parmetro con nombre nom y ste es
     * requerido. false en caso contrario.
     */
    public boolean isParamRequired(String nom) {
      JIPParameter p=null;
      boolean res=false;
      for(int i=0;i<params.size();i++) {
        p=(JIPParameter)params.elementAt(i);
        if(nom.equals(p.getName())&&p.isRequired()) {
          res=true;
          break;
        } 
      }
      return res;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Nos dice si un resultado de la funcin es requerido.
     * @param nom Nombre a verificar si es o no un resultado requerido.
     * @return true si la funcin tiene un resultado con nombre nom y ste es
     * requerido. false en caso contrario.
     */
    public boolean isResultRequired(String nom) {
      JIPParameter p=null;
      boolean res=false;
      for(int i=0;i<results.size();i++) {
        p=(JIPParameter)results.elementAt(i);
        if(nom.equals(p.getName())&&p.isRequired()) {
          res=true;
          break;
        }
      }
      return res;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Nos dice si un parmetro de la funcin tiene valor asignado.
     * @param nom Nombre a verificar si es o no un parmetro con valor asignado.
     * @return true si la funcin tiene un parmetro con nombre nom y ste tiene
     * valor asignado. false en caso contrario.
     */
    public boolean isParamAsigned(String nom) {
      JIPParameter p=null;
      boolean res=false;
      for(int i=0;i<params.size();i++) {
        p=(JIPParameter)params.elementAt(i);
        if(nom.equals(p.getName())&&p.isAsigned()) {
          res=true;
          break;
        } 
      }
      return res;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Nos dice si un resultado de la funcin tiene valor asignado.
     * @param nom Nombre a verificar si es o no un resultado con valor asignado.
     * @return true si la funcin tiene un resultado con nombre nom y ste tiene
     * valor asignado. false en caso contrario.
     */
    public boolean isResultAsigned(String nom) {
      JIPParameter p=null;
      boolean res=false;
      for(int i=0;i<results.size();i++) {
        p=(JIPParameter)results.elementAt(i);
        if(nom.equals(p.getName())&&p.isAsigned()) {
          res=true;
          break;
        }
      }
      return res;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el tipo de un parmetro de la funcin.
     * @param nom Nombre del parmetro del cual devolver su tipo.
     * @return Tipo del parmetro de nombre nom. -1 si el parmetro nom no existe.
     */
    public int getParamType(String nom) {
      JIPParameter p=null;
      int res=-1;
      for(int i=0;i<params.size();i++) {
        p=(JIPParameter)params.elementAt(i);
        if(nom.equals(p.getName())) {
          res=p.getType();
          break;
        } 
      }
      return res;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el tipo de un resultado de la funcin.
     * @param nom Nombre del resultado del cual devolver su tipo.
     * @return Tipo del resultado de nombre nom. -1 si el resultado nom no existe.
     */
    public int getResultType(String nom) {
      JIPParameter p=null;
      int res=-1;
      for(int i=0;i<results.size();i++) {
        p=(JIPParameter)results.elementAt(i);
        if(nom.equals(p.getName())) {
          res=p.getType();
          break;
        }
      }
      return res;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene la descripcin de un parmetro de la funcin.
     * @param nom Nombre del parmetro del cual devolver su descripcin.
     * @return Descripcin del parmetro de nombre nom. null si el parmetro nom
     * no existe.
     */
    public String getParamDescr(String nom) {
      JIPParameter p=null;
      String res=null;
      for(int i=0;i<params.size();i++) {
        p=(JIPParameter)params.elementAt(i);
        if(nom.equals(p.getName())) {
          res=p.getDescription();
          break;
        } 
      }
      return res;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene la descripcin de un resultado de la funcin.
     * @param nom Nombre del resultado del cual devolver su descripcin.
     * @return Descripcin del resultado de nombre nom. null si el resultado nom
     * no existe.
     */
    public String getResultDescr(String nom) {
      JIPParameter p=null;
      String res=null;
      for(int i=0;i<results.size();i++) {
        p=(JIPParameter)results.elementAt(i);
        if(nom.equals(p.getName())) {
          res=p.getDescription();
          break;
        }
      }
      return res;
    }

    //**********************************************
    // METODOS PARA ASIGNAR VALORES A LOS PARAMETROS
    //**********************************************

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna a un parmetro de la funcin un valor booleano.
     * @param nom Nombre del parmetro a asignar (debe existir y ser de tipo pBOOL)
     * @param v Valor a asignar.
     */
    public void setParamValue(String nom,boolean v) {
      JIPParameter p=null;
      for(int i=0;i<params.size();i++) {
        p=(JIPParameter)params.elementAt(i);
        if(nom.equals(p.getName())&&p.getType()==JIP.pBOOL) {
          p.setValue(v);
          break;
        } 
      }
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna a un parmetro de la funcin un valor entero.
     * @param nom Nombre del parmetro a asignar (debe existir y ser de tipo pINT)
     * @param v Valor a asignar.
     */
    public void setParamValue(String nom,int v) {
      JIPParameter p=null;
      for(int i=0;i<params.size();i++) {
        p=(JIPParameter)params.elementAt(i);
        if(nom.equals(p.getName())&&p.getType()==JIP.pINT) {
          p.setValue(v);
          break;
        } 
      }
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna a un parmetro de la funcin un valor real.
     * @param nom Nombre del parmetro a asignar (debe existir y ser de tipo pREAL)
     * @param v Valor a asignar.
     */
    public void setParamValue(String nom,float v) {
      JIPParameter p=null;
      for(int i=0;i<params.size();i++) {
        p=(JIPParameter)params.elementAt(i);
        if(nom.equals(p.getName())&&p.getType()==JIP.pREAL) {
          p.setValue(v);
          break;
        } 
      }
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna a un parmetro de la funcin un valor string.
     * @param nom Nombre del parmetro a asignar (debe existir y ser de tipo pSTRING)
     * @param v Valor a asignar.
     */
    public void setParamValue(String nom,String v) {
      JIPParameter p=null;
      for(int i=0;i<params.size();i++) {
        p=(JIPParameter)params.elementAt(i);
        if(nom.equals(p.getName())&&p.getType()==JIP.pSTRING) {
          p.setValue(new String(v));
          break;
        } 
      }
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna a un parmetro de la funcin un valor imagen.
     * @param nom Nombre del parmetro a asignar (debe existir y ser de tipo pIMAGE)
     * @param img Imagen a asignar.
     */
    public void setParamValue(String nom,JIPImage img) {
      JIPParameter p=null;
      for(int i=0;i<params.size();i++) {
        p=(JIPParameter)params.elementAt(i);
        if(nom.equals(p.getName())&&p.getType()==JIP.pIMAGE) {
          p.setValue(img);
          break;
        }
      }
    }

    //***************************************************
    // METODOS PARA OBTENER LOS VALORES DE LOS PARAMETROS
    //***************************************************
     
    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el valor booleano asignado a un parmetro de la funcin.
     * @param nom Nombre del parmetro (debe existir y ser de tipo pBOOL)
     * @return Valor asignado al parmetro. Si el parmetro no est asignado
     * y no es requerido se devuelve el valor asignado por defecto.
     */
    public boolean getParamValueBool(String nom) {
      JIPParameter p=null;
      boolean res=false;
      for(int i=0;i<params.size();i++) {
        p=(JIPParameter)params.elementAt(i);
        if(nom.equals(p.getName())&&p.getType()==JIP.pBOOL) {
          res=p.getValueBool();
          break;
        } 
      }
      return res;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el valor entero asignado a un parmetro de la funcin.
     * @param nom Nombre del parmetro (debe existir y ser de tipo pINT)
     * @return Valor asignado al parmetro. Si el parmetro no est asignado
     * y no es requerido se devuelve el valor asignado por defecto.
     */
    public int getParamValueInt(String nom) {
      JIPParameter p=null;
      int res=0;
      for(int i=0;i<params.size();i++) {
        p=(JIPParameter)params.elementAt(i);
        if(nom.equals(p.getName())&&p.getType()==JIP.pINT) {
          res=p.getValueInt();
          break;
        } 
      }
      return res;
    }

		 /**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el valor real asignado a un parmetro de la funcin.
     * @param nom Nombre del parmetro (debe existir y ser de tipo pREAL)
     * @return Valor asignado al parmetro. Si el parmetro no est asignado
     * y no es requerido se devuelve el valor asignado por defecto.
     */
    public float getParamValueReal(String nom) {
      JIPParameter p=null;
      float res=0;
      for(int i=0;i<params.size();i++) {
        p=(JIPParameter)params.elementAt(i);
        if(nom.equals(p.getName())&&p.getType()==JIP.pREAL) {
          res=p.getValueReal();
          break;
        } 
      }
      return res;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el valor string asignado a un parmetro de la funcin.
     * @param nom Nombre del parmetro (debe existir y ser de tipo pSTRING)
     * @return Valor asignado al parmetro. Si el parmetro no est asignado
     * y no es requerido se devuelve el valor asignado por defecto.
     */
    public String getParamValueStr(String nom) {
      JIPParameter p=null;
      String res=null;
      for(int i=0;i<params.size();i++) {
        p=(JIPParameter)params.elementAt(i);
        if(nom.equals(p.getName())&&p.getType()==JIP.pSTRING) {
          res=p.getValueStr();
          break;
        } 
      }
      return res;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el valor imagen asignado a un parmetro de la funcin.
     * @param nom Nombre del parmetro (debe existir y ser de tipo pIMAGE)
     * @return Valor asignado al parmetro. Si el parmetro no est asignado
     * y no es requerido se devuelve el valor asignado por defecto.
     */
    public JIPImage getParamValueImg(String nom) {
      JIPParameter p=null;
      JIPImage res=null;
      for(int i=0;i<params.size();i++) {
        p=(JIPParameter)params.elementAt(i);
        if(nom.equals(p.getName())&&p.getType()==JIP.pIMAGE) {
          res=p.getValueImg();
          break;
        }
      }
      return res;
    }

    //**********************************************
    // METODOS PARA ASIGNAR VALORES A LOS RESULTADOS
    //**********************************************

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna a un resultado de la funcin un valor booleano.
     * @param nom Nombre del resultado a asignar (debe existir y ser de tipo pBOOL)
     * @param v Valor a asignar.
     */
    public void setResultValue(String nom,boolean v) {
      JIPParameter p=null;
      for(int i=0;i<results.size();i++) {
        p=(JIPParameter)results.elementAt(i);
        if(nom.equals(p.getName())&&p.getType()==JIP.pBOOL) {
          p.setValue(v);
          break;
        }
      }
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna a un resultado de la funcin un valor entero.
     * @param nom Nombre del resultado a asignar (debe existir y ser de tipo pINT)
     * @param v Valor a asignar.
     */
    public void setResultValue(String nom,int v) {
      JIPParameter p=null;
      for(int i=0;i<results.size();i++) {
        p=(JIPParameter)results.elementAt(i);
        if(nom.equals(p.getName())&&p.getType()==JIP.pINT) {
          p.setValue(v);
          break;
        }
      }
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna a un resultado de la funcin un valor real.
     * @param nom Nombre del resultado a asignar (debe existir y ser de tipo pREAL)
     * @param v Valor a asignar.
     */
    public void setResultValue(String nom,float v) {
      JIPParameter p=null;
      for(int i=0;i<results.size();i++) {
        p=(JIPParameter)results.elementAt(i);
        if(nom.equals(p.getName())&&p.getType()==JIP.pREAL) {
          p.setValue(v);
          break;
        }
      }
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna a un resultado de la funcin un valor string.
     * @param nom Nombre del resultado a asignar (debe existir y ser de tipo pSTRING)
     * @param v Valor a asignar.
     */
    public void setResultValue(String nom,String v) {
      JIPParameter p=null;
      for(int i=0;i<results.size();i++) {
        p=(JIPParameter)results.elementAt(i);
        if(nom.equals(p.getName())&&p.getType()==JIP.pSTRING) {
          p.setValue(new String(v));
          break;
        }
      }
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna a un resultado de la funcin un valor imagen.
     * @param nom Nombre del resultado a asignar (debe existir y ser de tipo pIMAGE)
     * @param img Imagen a asignar.
     */
    public void setResultValue(String nom,JIPImage img) {
      JIPParameter p=null;
      for(int i=0;i<results.size();i++) {
        p=(JIPParameter)results.elementAt(i);
        if(nom.equals(p.getName())&&p.getType()==JIP.pIMAGE) {
          p.setValue(img);
          break;
        }
      }
    }

    //***************************************************
    // METODOS PARA OBTENER LOS VALORES DE LOS RESULTADOS
    //***************************************************

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el valor booleano asignado a un resultado de la funcin.
     * @param nom Nombre del resultado (debe existir y ser de tipo pBOOL)
     * @return Valor asignado al resultado. Si el resultado no est asignado
     * y no es requerido se devuelve el valor asignado por defecto.
     */
    public boolean getResultValueBool(String nom) {
      JIPParameter p=null;
      boolean res=false;
      for(int i=0;i<results.size();i++) {
        p=(JIPParameter)results.elementAt(i);
        if(nom.equals(p.getName())&&p.getType()==JIP.pBOOL) {
          res=p.getValueBool();
          break;
        }
      }
      return res;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el valor entero asignado a un resultado de la funcin.
     * @param nom Nombre del resultado (debe existir y ser de tipo pINT)
     * @return Valor asignado al resultado. Si el resultado no est asignado
     * y no es requerido se devuelve el valor asignado por defecto.
     */
    public int getResultValueInt(String nom) {
      JIPParameter p=null;
      int res=0;
      for(int i=0;i<results.size();i++) {
        p=(JIPParameter)results.elementAt(i);
        if(nom.equals(p.getName())&&p.getType()==JIP.pINT) {
          res=p.getValueInt();
          break;
        }
      }
      return res;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el valor real asignado a un resultado de la funcin.
     * @param nom Nombre del resultado (debe existir y ser de tipo pREAL)
     * @return Valor asignado al resultado. Si el resultado no est asignado
     * y no es requerido se devuelve el valor asignado por defecto.
     */
    public float getResultValueReal(String nom) {
      JIPParameter p=null;
      float res=0;
      for(int i=0;i<results.size();i++) {
        p=(JIPParameter)results.elementAt(i);
        if(nom.equals(p.getName())&&p.getType()==JIP.pREAL) {
          res=p.getValueReal();
          break;
        }
      }
      return res;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el valor string asignado a un resultado de la funcin.
     * @param nom Nombre del resultado (debe existir y ser de tipo pSTRING)
     * @return Valor asignado al resultado. Si el resultado no est asignado
     * y no es requerido se devuelve el valor asignado por defecto.
     */
    public String getResultValueStr(String nom) {
      JIPParameter p=null;
      String res=null;
      for(int i=0;i<results.size();i++) {
        p=(JIPParameter)results.elementAt(i);
        if(nom.equals(p.getName())&&p.getType()==JIP.pSTRING) {
          res=p.getValueStr();
          break;
        }
      }
      return res;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el valor imagen asignado a un resultado de la funcin.
     * @param nom Nombre del resultado (debe existir y ser de tipo pIMAGE)
     * @return Valor asignado al resultado. Si el resultado no est asignado
     * y no es requerido se devuelve el valor asignado por defecto.
     */
    public JIPImage getResultValueImg(String nom) {
      JIPParameter p=null;
      JIPImage res=null;
      for(int i=0;i<results.size();i++) {
        p=(JIPParameter)results.elementAt(i);
        if(nom.equals(p.getName())&&p.getType()==JIP.pIMAGE) {
          res=p.getValueImg();
          break;
        }
      }
      return res;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Chequea si todos los parmetros requeridos de la funcin tienen valor
     * asignado.
     * @return true si todos los parmetros requeridos de la funcin tienen valor
     * asignado o si la funcin no tiene parmetros. false en caso contrario.
     */
   public boolean paramsOK() {
      JIPParameter p=null;
      boolean ok=true;
      for(int i=0;i<params.size();i++) {
        p=(JIPParameter)params.elementAt(i);
        if(p.isRequired()&&(!p.isAsigned())) {
          ok=false;
          break;
        } 
      }
      return ok;
   }

}